=== modified file 'AUTHORS'
--- AUTHORS	2006-09-30 10:21:43 +0000
+++ AUTHORS	2006-10-14 19:14:14 +0000
@@ -1,8 +1,8 @@
 bzr-gtk is a plugin that is based on a collection of earlier plugins for 
-Bazaar. It is currently maintained by Jelmer Vernooij <jelmer@samba.org>.
+Bazaar. It is currently maintained by Jelmer Vernooij <jelmer@samba.org> and 
+Szilveszter Farkas <Szilveszter.Farkas@gmail.com>.
 
 bzrk was written by Scott James Remnant <scott@ubuntu.com>. 
-
 It's vaguely based on a screenshot of gitk (the equivalent tool for git)
 shown to me by Fabio M. Di Nitto <fabbione@ubuntu.com>.
 
@@ -13,4 +13,5 @@
 
 Olive was written by Szilveszter Farkas <Szilveszter.Farkas@gmail.com>.
 
-See the Bazaar history of this branch for a full list of authors.
+Numerous other people have contributed useful patches.  See the Bazaar history 
+of this branch for a full list of authors.

=== modified file 'nautilus-bzr.py'
--- nautilus-bzr.py	2006-10-08 21:57:40 +0000
+++ nautilus-bzr.py	2006-10-15 20:15:05 +0000
@@ -157,6 +157,42 @@
 
         return
 
+    def pull_cb(self, menu, vfs_file):
+        # We can only cope with local files
+        if vfs_file.get_uri_scheme() != 'file':
+            return
+
+        file = vfs_file.get_uri()
+
+        # We only want to continue here if we get a NotBranchError
+        try:
+            tree, path = WorkingTree.open_containing(file)
+        except NotBranchError:
+            return
+
+        from bzrlib.plugins.gtk.olive.pull import PullDialog
+        dialog = PullDialog(tree, path)
+        dialog.display()
+        gtk.main()
+
+    def merge_cb(self, menu, vfs_file):
+        # We can only cope with local files
+        if vfs_file.get_uri_scheme() != 'file':
+            return
+
+        file = vfs_file.get_uri()
+
+        # We only want to continue here if we get a NotBranchError
+        try:
+            tree, path = WorkingTree.open_containing(file)
+        except NotBranchError:
+            return
+
+        from bzrlib.plugins.gtk.olive.merge import MergeDialog
+        dialog = MergeDialog(tree, path)
+        dialog.display()
+        gtk.main()
+
     def get_background_items(self, window, vfs_file):
         items = []
         file = vfs_file.get_uri()
@@ -183,6 +219,18 @@
         item.connect('activate', self.log_cb, vfs_file)
         items.append(item)
 
+        item = nautilus.MenuItem('BzrNautilus::pull',
+                             'Pull',
+                             'Pull from another branch')
+        item.connect('activate', self.pull_cb, vfs_file)
+        items.append(item)
+
+        item = nautilus.MenuItem('BzrNautilus::merge',
+                             'Merge',
+                             'Merge from another branch')
+        item.connect('activate', self.merge_cb, vfs_file)
+        items.append(item)
+
         item = nautilus.MenuItem('BzrNautilus::commit',
                              'Commit',
                              'Commit Changes')

=== modified file 'olive.glade'
--- olive.glade	2006-10-14 10:26:00 +0000
+++ olive.glade	2006-10-19 13:06:30 +0000
@@ -670,8 +670,8 @@
 </widget>
 
 <widget class="GtkDialog" id="window_commit">
-  <property name="width_request">400</property>
-  <property name="height_request">400</property>
+  <property name="width_request">500</property>
+  <property name="height_request">500</property>
   <property name="title" translatable="yes">Olive - Commit</property>
   <property name="type">GTK_WINDOW_TOPLEVEL</property>
   <property name="window_position">GTK_WIN_POS_NONE</property>
@@ -802,7 +802,7 @@
 	<widget class="GtkVPaned" id="vpaned_commit">
 	  <property name="visible">True</property>
 	  <property name="can_focus">True</property>
-	  <property name="position">120</property>
+	  <property name="position">250</property>
 
 	  <child>
 	    <widget class="GtkVBox" id="vbox_commit_select">
@@ -864,6 +864,61 @@
 		  <property name="fill">True</property>
 		</packing>
 	      </child>
+
+	      <child>
+		<widget class="GtkLabel" id="label_commit_pending">
+		  <property name="visible">True</property>
+		  <property name="label" translatable="yes">Pending merges:</property>
+		  <property name="use_underline">False</property>
+		  <property name="use_markup">False</property>
+		  <property name="justify">GTK_JUSTIFY_LEFT</property>
+		  <property name="wrap">False</property>
+		  <property name="selectable">False</property>
+		  <property name="xalign">0</property>
+		  <property name="yalign">0.5</property>
+		  <property name="xpad">0</property>
+		  <property name="ypad">0</property>
+		  <property name="ellipsize">PANGO_ELLIPSIZE_NONE</property>
+		  <property name="width_chars">-1</property>
+		  <property name="single_line_mode">False</property>
+		  <property name="angle">0</property>
+		</widget>
+		<packing>
+		  <property name="padding">0</property>
+		  <property name="expand">False</property>
+		  <property name="fill">False</property>
+		</packing>
+	      </child>
+
+	      <child>
+		<widget class="GtkScrolledWindow" id="scrolledwindow_commit_pending">
+		  <property name="visible">True</property>
+		  <property name="can_focus">True</property>
+		  <property name="hscrollbar_policy">GTK_POLICY_AUTOMATIC</property>
+		  <property name="vscrollbar_policy">GTK_POLICY_AUTOMATIC</property>
+		  <property name="shadow_type">GTK_SHADOW_NONE</property>
+		  <property name="window_placement">GTK_CORNER_TOP_LEFT</property>
+
+		  <child>
+		    <widget class="GtkTreeView" id="treeview_commit_pending">
+		      <property name="visible">True</property>
+		      <property name="can_focus">True</property>
+		      <property name="headers_visible">True</property>
+		      <property name="rules_hint">False</property>
+		      <property name="reorderable">False</property>
+		      <property name="enable_search">True</property>
+		      <property name="fixed_height_mode">False</property>
+		      <property name="hover_selection">False</property>
+		      <property name="hover_expand">False</property>
+		    </widget>
+		  </child>
+		</widget>
+		<packing>
+		  <property name="padding">0</property>
+		  <property name="expand">True</property>
+		  <property name="fill">True</property>
+		</packing>
+	      </child>
 	    </widget>
 	    <packing>
 	      <property name="shrink">True</property>
@@ -5514,7 +5569,7 @@
 	      <signal name="clicked" handler="on_button_merge_cancel_clicked" last_modification_time="Mon, 09 Oct 2006 13:35:13 GMT"/>
 	    </widget>
 	  </child>
-	  
+
 	  <child>
 	    <widget class="GtkButton" id="button_merge_merge">
 	      <property name="visible">True</property>
@@ -5590,7 +5645,7 @@
 		</widget>
 	      </child>
 	    </widget>
-	  </child>	  
+	  </child>
 	</widget>
 	<packing>
 	  <property name="padding">0</property>

=== modified file 'olive/__init__.py'
--- olive/__init__.py	2006-10-14 11:44:16 +0000
+++ olive/__init__.py	2006-10-19 13:06:30 +0000
@@ -650,6 +650,7 @@
     def set_sensitivity(self):
         """ Set menu and toolbar sensitivity. """
         self.menuitem_branch_init.set_sensitive(self.notbranch)
+        self.menuitem_branch_get.set_sensitive(self.notbranch)
         self.menuitem_branch_checkout.set_sensitive(self.notbranch)
         self.menuitem_branch_pull.set_sensitive(not self.notbranch)
         self.menuitem_branch_push.set_sensitive(not self.notbranch)

=== modified file 'olive/commit.py'
--- olive/commit.py	2006-10-04 19:11:46 +0000
+++ olive/commit.py	2006-10-15 17:47:36 +0000
@@ -44,6 +44,8 @@
         self.checkbutton_local = self.glade.get_widget('checkbutton_commit_local')
         self.textview = self.glade.get_widget('textview_commit')
         self.file_view = self.glade.get_widget('treeview_commit_select')
+        self.pending_label = self.glade.get_widget('label_commit_pending')
+        self.pending_view = self.glade.get_widget('treeview_commit_pending')
 
         file_id = self.wt.path2id(wtpath)
 
@@ -56,6 +58,9 @@
         self.old_tree = self.wt.branch.repository.revision_tree(self.wt.branch.last_revision())
         self.delta = self.wt.changes_from(self.old_tree)
         
+        # Get pending merges
+        self.pending = self._pending_merges(self.wt)
+        
         # Dictionary for signal_autoconnect
         dic = { "on_button_commit_commit_clicked": self.commit,
                 "on_button_commit_cancel_clicked": self.close }
@@ -65,6 +70,8 @@
         
         # Create the file list
         self._create_file_view()
+        # Create the pending merges
+        self._create_pending_merges()
     
     def display(self):
         """ Display the Push dialog. """
@@ -77,6 +84,13 @@
                 # we have a checkout, so the local commit checkbox must appear
                 self.checkbutton_local.show()
             
+            if self.pending:
+                # There are pending merges, file selection not supported
+                self.file_view.set_sensitive(False)
+            else:
+                # No pending merges
+                self.pending_view.set_sensitive(False)
+            
             self.textview.modify_font(pango.FontDescription("Monospace"))
             self.window.show()
             
@@ -108,6 +122,27 @@
         for path, id, kind, text_modified, meta_modified in self.delta.modified:
             self.file_store.append([ True, path, _('modified') ])
     
+    def _create_pending_merges(self):
+        liststore = gtk.ListStore(gobject.TYPE_STRING,
+                                  gobject.TYPE_STRING,
+                                  gobject.TYPE_STRING)
+        self.pending_view.set_model(liststore)
+        
+        self.pending_view.append_column(gtk.TreeViewColumn(_('Date'),
+                                        gtk.CellRendererText(), text=0))
+        self.pending_view.append_column(gtk.TreeViewColumn(_('Committer'),
+                                        gtk.CellRendererText(), text=1))
+        self.pending_view.append_column(gtk.TreeViewColumn(_('Summary'),
+                                        gtk.CellRendererText(), text=2))
+        
+        if not self.pending:
+            return
+        
+        for item in self.pending:
+            liststore.append([ item['date'],
+                               item['committer'],
+                               item['summary'] ])
+    
     def _get_specific_files(self):
         ret = []
         it = self.file_store.get_iter_first()
@@ -122,6 +157,70 @@
         model[path][0] = not model[path][0]
         return
     
+    def _pending_merges(self, wt):
+        """ Return a list of pending merges or None if there are none of them. """
+        parents = wt.get_parent_ids()
+        if len(parents) < 2:
+            return None
+        
+        import re
+        from bzrlib.osutils import format_date
+        
+        pending = parents[1:]
+        branch = wt.branch
+        last_revision = parents[0]
+        
+        if last_revision is not None:
+            try:
+                ignore = set(branch.repository.get_ancestry(last_revision))
+            except errors.NoSuchRevision:
+                # the last revision is a ghost : assume everything is new 
+                # except for it
+                ignore = set([None, last_revision])
+        else:
+            ignore = set([None])
+        
+        pm = []
+        for merge in pending:
+            ignore.add(merge)
+            try:
+                m_revision = branch.repository.get_revision(merge)
+                
+                rev = {}
+                rev['committer'] = re.sub('<.*@.*>', '', m_revision.committer).strip(' ')
+                rev['summary'] = m_revision.get_summary()
+                rev['date'] = format_date(m_revision.timestamp,
+                                          m_revision.timezone or 0, 
+                                          'original', date_fmt="%Y-%m-%d",
+                                          show_offset=False)
+                
+                pm.append(rev)
+                
+                inner_merges = branch.repository.get_ancestry(merge)
+                assert inner_merges[0] is None
+                inner_merges.pop(0)
+                inner_merges.reverse()
+                for mmerge in inner_merges:
+                    if mmerge in ignore:
+                        continue
+                    mm_revision = branch.repository.get_revision(mmerge)
+                    
+                    rev = {}
+                    rev['committer'] = re.sub('<.*@.*>', '', mm_revision.committer).strip(' ')
+                    rev['summary'] = mm_revision.get_summary()
+                    rev['date'] = format_date(mm_revision.timestamp,
+                                              mm_revision.timezone or 0, 
+                                              'original', date_fmt="%Y-%m-%d",
+                                              show_offset=False)
+                
+                    pm.append(rev)
+                    
+                    ignore.add(mmerge)
+            except errors.NoSuchRevision:
+                print "DEBUG: NoSuchRevision:", merge
+        
+        return pm
+
     def commit(self, widget):
         textbuffer = self.textview.get_buffer()
         start, end = textbuffer.get_bounds()
@@ -130,7 +229,10 @@
         checkbutton_strict = self.glade.get_widget('checkbutton_commit_strict')
         checkbutton_force = self.glade.get_widget('checkbutton_commit_force')
         
-        specific_files = self._get_specific_files()
+        if not self.pending:
+            specific_files = self._get_specific_files()
+        else:
+            specific_files = None
         
         try:
             self.wt.commit(message, 

=== modified file 'olive/merge.py'
--- olive/merge.py	2006-10-09 21:21:47 +0000
+++ olive/merge.py	2006-10-17 23:47:33 +0000
@@ -25,10 +25,11 @@
 import gtk
 import gtk.glade
 
+from bzrlib.branch import Branch
 import bzrlib.errors as errors
 
 from __init__ import gladefile
-from dialog import error_dialog
+from dialog import error_dialog, info_dialog, warning_dialog
 
 class MergeDialog:
     """ Display the Merge dialog and perform the needed actions. """
@@ -48,16 +49,56 @@
 
         self.wt = wt
         self.wtpath = wtpath
+        
+        # Get some widgets
+        self.entry = self.glade.get_widget('entry_merge')
 
     def display(self):
         """ Display the Add file(s) dialog. """
         self.window.show_all()
 
     def merge(self, widget):
-        print "DEBUG: Merge button pressed."
+        branch = self.entry.get_text()
+        if branch == "":
+            error_dialog(_('Branch not given'),
+                         _('Please specify a branch to merge from.'))
+            return
+
+        try:
+            other_branch = Branch.open_containing(branch)[0]
+        except errors.NotBranchError:
+            error_dialog(_('Specified location not a branch'),
+                         _('Please specify a branch you want to merge from.'))
+            return
+        
+        try:
+            conflicts = self.wt.merge_from_branch(other_branch)
+        except errors.BzrCommandError, errmsg:
+            error_dialog(_('Bazaar command error'), str(errmsg))
+            return
+        
+        self.close()
+        if conflicts == 0:
+            # No conflicts found.
+            info_dialog(_('Merge successful'),
+                        _('All changes applied successfully.'))
+        else:
+            # There are conflicts to be resolved.
+            warning_dialog(_('Conflicts encountered'),
+                           _('Please resolve the conflicts manually before committing.'))
     
     def open(self, widget):
-        print "DEBUG: Open branch button pressed."
-    
+        fcd = gtk.FileChooserDialog(title="Please select a folder",
+                                    parent=self.window,
+                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                                             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        fcd.set_default_response(gtk.RESPONSE_OK)
+        
+        if fcd.run() == gtk.RESPONSE_OK:
+            self.entry.set_text(fcd.get_filename())
+        
+        fcd.destroy()
+        
     def close(self, widget=None):
         self.window.destroy()

